Module _unsafe

Module _unsafe 

Contains some of the unsafe backing types used by interned, mainly Static.

Structs

StaticOsStr
An internal struct used to represent a type-erased, heap-allocated &'static OsStr.
StaticPath
An internal struct used to represent a type-erased, heap-allocated &'static Path.
StaticSlice
An unsafe internal struct used to represent a type-erased, heap-allocated, static slice (i.e. not a value or reference).
StaticStr
An internal struct used to represent a type-erased, heap-allocated, static string (&'static str).
StaticValue
An unsafe internal struct used to represent a type-erased, heap-allocated, static value (i.e. not a reference or slice).

Enums

Static
An (unsafe) internal enum that generalizes over StaticValue, StaticSlice, StaticOsStr, StaticPath, and StaticStr.