Contains some of the unsafe backing types used by interned, mainly Static.
Structs
- Static
OsStr - An internal struct used to represent a type-erased, heap-allocated
&'static OsStr. - Static
Path - An internal struct used to represent a type-erased, heap-allocated
&'static Path. - Static
Slice - An unsafe internal struct used to represent a type-erased, heap-allocated, static slice (i.e. not a value or reference).
- Static
Str - An internal struct used to represent a type-erased, heap-allocated, static string
(
&'static str). - Static
Value - An unsafe internal struct used to represent a type-erased, heap-allocated, static value (i.e. not a reference or slice).
Enums
- Static
- An (unsafe) internal enum that generalizes over
StaticValue,StaticSlice,StaticOsStr,StaticPath, andStaticStr.